function aufftdemo(action)
% an interactive audio FFT demo

if ~ischar(action)
  % install the sound
  wav = action;
  action = 'Initialize';
else
  % get the object we're using to remember our state
  h = get(gcbf,'UserData');
end

switch action
 case 'Initialize'
  % create the main figure
  h.figh = figure('name', 'Audio FFT Demo', ...
		 'numbertitle','off','Position',[80 0 950 700],...
		 'units', 'pixels', 'doublebuffer', 'on');
  h.Axis1 = subplot(4, 1, 1);
  h.len = length(wav);
  h.wav = double(wav);
  h.Plot1 = plot((1:h.len)/8000, wav);
  ylabel('Amplitude')
  xlabel('Time')
  set(h.Plot1, 'buttondownfcn', 'aufftdemo play1');
  h.SumSq = sum(double(wav(:)).^2);
  
  h.Axis2 = subplot(4,1,2);
  h.Fau = fftshift(fft(wav));
  aFau = abs(h.Fau);
  h.sFau = -sort(-aFau(:));
  h.plot2 = plot(((1:h.len)-1-h.len/2)*8000/h.len, abs(h.Fau));
  ylabel('Magnitude')
  xlabel('Frequency')
  
  h.Axis3 = subplot(4,1,3);
  h.Mask = zeros(h.len, 1);
  h.plot3 = plot(((1:h.len)-1-h.len/2)*8000/h.len, h.Mask);
  ylabel('Mask');
  xlabel('Frequency')
  
  h.Axis4 = subplot(4,1,4);
  h.plot4 = plot((1:h.len)/8000, h.Mask);
  set(h.plot4, 'buttondownfcn', 'aufftdemo play2');
  ylabel('Amplitude')
  xlabel('Time')
  
  h.PCntSlider = uicontrol(gcf, ...
			   'Style', 'slider', ...
			   'Position', [35 400 20 250], ...
			   'Min', 0, 'Max', 1, 'Value', 0, ...
			   'SliderStep', [0.0001 0.01 ], ...
			   'CallBack', 'aufftdemo Slider');
  uicontrol(gcf, ...
	    'Style', 'text', ...
	    'Position', [15 370 60 20], ...
	    'String', 'Threshold');
  
  h.Error = uicontrol(gcf, ...
		      'Style', 'text', ...
		      'Position', [300 10 40 20], ...
		      'String', 'Error %');
  uicontrol(gcf, ...
	    'Style', 'text', ...
	    'Position', [220 10 60 20], ...
	    'String', 'RMS Error');
  
  h.MaskPercent = uicontrol(gcf, ...
		      'Style', 'text', ...
		      'Position', [600 10 40 20], ...
		      'String', 'On %');
  uicontrol(gcf, ...
	    'Style', 'text', ...
	    'Position', [520 10 60 20], ...
	    'String', '% Selected')
  
  set(gcf, 'UserData', h);
  
  
 case 'Slider'
  val = get(gco, 'Value');
  val = h.sFau(fix(val * (length(h.sFau) - 1)) + 1);
  h.Mask = abs(h.Fau) >= val - 1e-8;
  % make sure the mask is symmetric about 0
  h.Mask((h.len/2+1):h.len) = h.Mask((h.len/2+1):h.len) | h.Mask((h.len/2+1):-1:2);
  h.Mask((h.len/2+1):-1:2) = h.Mask((h.len/2+1):-1:2) | h.Mask((h.len/2+1):h.len);
  
  localUpdate(h);
  
 case 'play1'
  sound(h.wav, 8000);
  
 case 'play2'
  sound(h.nau, 8000);
  
end

function localUpdate(h)
  subplot(4,1,3);
  set(h.plot3, 'YData', h.Mask .* abs(h.Fau));
  %axis([-h.len/2 h.len/2-1 -0.1 1.1 ]);
  set(h.MaskPercent, 'String', [num2str(100*sum(h.Mask(:))/length(h.Mask(:))) '%']);
  
  h.nau = real(ifft(ifftshift(h.Fau .* h.Mask)));
  subplot(4,1,4);
  set(h.plot4, 'YData', h.nau);
  axis([1/8000 h.len/8000 -1 1]);
  err = h.wav - h.nau;
  set(h.Error, 'String', [num2str(100*sqrt(sum(err(:).^2)/h.SumSq)) '%']);
  
  set(gcf, 'UserData', h);
  
  
